# _*_coding:utf-8_*_
from django.conf import settings
from django.conf.urls import static
from django.conf.urls import url
from django.urls import re_path

from .apis.resource_upload_file import UploadFile
from .apis.resource_upload_image import UploadImage

# 应用名称
app_name = 'resource'

urlpatterns = [
    url(r'upload_file/?$', UploadFile.as_view(), name='st_upload_file'),  # 文件上传
    url(r'upload_image/?$', UploadImage.as_view(), name='st_upload_image'),  # 图片上传
    # re_path(r'^_upload_image/?$', UploadImage.as_view(), ),

    # 这里要填写/static/和/media/路径，否则django不会返回静态文件。
    re_path("static/(?P<path>.*)$", static.serve, {"document_root": settings.STATIC_ROOT, "show_indexes": False}, "static"),
    re_path("media/(?P<path>.*)$", static.serve, {"document_root": settings.MEDIA_ROOT, "show_indexes": False}, "media"),
]



from django.db import models
# from apps.user.models import BaseInfo
# Create your models here.


class ResourceImage(models.Model):
    class Meta:
        db_table = 'resource_image'
        verbose_name = '资源 - 图片表'
        verbose_name_plural = '1. 资源 - 图片表'

    user_id = models.BigIntegerField(verbose_name='用户', db_column='user_id', db_index=True)
    title = models.CharField(verbose_name='图片标题', max_length=255, blank=True, null=True)
    url = models.CharField(verbose_name='图片链接', max_length=255, blank=True, null=True, db_index=True)
    filename = models.CharField(verbose_name='文件名', max_length=255, blank=True, null=True)
    format = models.CharField(verbose_name='文件类型', max_length=32)
    thumb = models.TextField(verbose_name='缩略图', blank=True, null=True)
    md5 = models.CharField(verbose_name='MD5', max_length=255, blank=True, null=True) # 判断使文件是否有效且唯一。
    # thumb = models.ImageField(verbose_name='缩略图', upload_to="static/images", max_length=21845, blank=True, null=True, help_text='缩略图')
    snapshot = models.JSONField(verbose_name='快照', blank=True, null=True)

    def __str__(self):
        return f"{self.id}"


class ResourceImageMap(models.Model):
    class Meta:
        db_table = 'resource_image_map'
        verbose_name_plural = '3. 资源 - 图片映射表'

    image_id = models.ForeignKey(verbose_name='图片ID', to=ResourceImage, related_name='image_id_set+', on_delete=models.DO_NOTHING, db_column='image_id')
    source_id = models.BigIntegerField(verbose_name='来源ID', blank=True, null=True)
    source_table = models.CharField(verbose_name='来源表', max_length=128, blank=True, null=True)
    price = models.DecimalField(verbose_name='价格', max_digits=32, decimal_places=8, blank=True, null=True, db_index=True)

    def __str__(self):
        return f"{self.id}"



#_*_coding:utf-8_*_
from django.urls import re_path
from django.conf.urls import static, url
from django.conf import settings


from apps.resource.apis import resource_upload_image


urlpatterns = [
    re_path(r'^_upload_image/?$', resource_upload_image.UploadImage.as_view(), ),

    # 这里要填写/static/和/media/路径，否则django不会返回静态文件。
    re_path(
        "static/(?P<path>.*)$",
        static.serve,
        {"document_root": settings.STATIC_ROOT, "show_indexes": False},
        "static"
    ),
    re_path(
        "media/(?P<path>.*)$",
        static.serve,
        {"document_root": settings.MEDIA_ROOT, "show_indexes": False},
        "media"
    ),
]



# encoding: utf-8
"""
@project: djangoModel->resource_upload_file
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 文件上传接口
@created_time: 2022/8/2 10:28
"""
from django.core.paginator import Paginator
from rest_framework.views import APIView

from xj_resource.models import ResourceFile
from xj_resource.services.upload_file_service import UploadFileService
from xj_resource.utils.custom_response import util_response
from xj_resource.utils.model_handle import only_filed_handle, parse_model


class UploadFile(APIView):
    def post(self, request):
        user_id = request.user.id or 0
        file = request.FILES.get('file')
        title = request.POST.get('title', '')
        group_id = request.POST.get('group_id', 0)

        uploader = UploadFileService(input_file=file)
        info = uploader.info_detail()
        # 获取信息，并把非素材上传信息补全
        info['title'] = title or info['snapshot']['old_filename']
        info['user_id'] = user_id
        info['group_id'] = group_id
        # 写入磁盘
        uploader.write_disk()
        # 写入数据库
        image_instance = uploader.save_to_db(info)
        if image_instance:
            info['id'] = image_instance.id
        if not uploader.is_valid():
            return util_response(err=4003, msg=uploader.get_error_message())
        return util_response(data=info)

    # 文件列表
    def get(self, request):
        params = request.query_params.copy()
        limit = params.pop('limit', 20)
        page = params.pop('page', 20)
        params = only_filed_handle(params, {
            "title": "title__contains",
            "filename": "filename_contains",
            "md5": "md5",
            "user_id": "user_id"
        }, None)
        list_obj = ResourceFile.objects.filter(**params)
        count = list_obj.count()
        res_set = Paginator(list_obj, limit).get_page(page)
        return util_response(data={'count': count, 'page': page, 'limit': limit, "list": parse_model(res_set)})



import os
import re
from rest_framework import serializers
from rest_framework.response import Response
from rest_framework.views import APIView

# from apps.user.services import UserService
from main.settings import STATICFILES_DIRS
from main.settings import STATIC_URL
from ..utils.digit_algorithm import DigitAlgorithm
from ..utils.file_operate import FileOperate
from ..models import *


# 声明序列化
class UploadImageSerializer(serializers.ModelSerializer):
    class Meta:
        model = ResourceImage
        # 序列化验证检查，检查必填项的字段
        fields = ['id', 'user_id', 'title', 'url', 'filename', 'format', 'md5']


class UploadImage(APIView):
    params = None
    serializer_params = None

    print("-" * 30, os.path.basename(__file__), "-" * 30)

    # POST方法，上传图片
    def post(self, request, *args, **kwargs):
        # param = self.params = request.query_params  # 返回QueryDict类型
        param = self.params = request.data  # 返回QueryDict类型
        item = self.serializer_params = {}  # 将要写入的某条数据
        print(">>> param", param)

        # ========== 一、检查：验证权限 ==========
        token = self.request.META.get('HTTP_AUTHORIZATION', '')
        if not token:
            return Response({'err': 6001, 'msg': '缺少Token', })
        user_id = UserService.checkToken(token)
        item['user_id'] = user_id
        if not user_id:
            return Response({'err': 6002, 'msg': 'token验证失败', })

        # ========== 二、检查：必填性 ==========
        # 对应postman的Body的key=file，value=上传文件的名称 watermelonhh.jpg
        input_file = request.FILES.get("image")
        if input_file is None:
            return Response({'err': 2001, 'msg': '未选择上传图片', })

        # ========== 三、检查：内容的准确性 ==========
        ret = re.search(r'(.*)\.(\w{3,4})$', input_file.name)
        if not ret:
            return Response({'err': 3001, 'msg': '上传的文件名不合法', })
        input_filename = item['filename'] = ret.group(1)
        format = item['format'] = ret.group(2)

        if format not in ('jpg', 'jpeg', 'gif', 'png', 'svg'):
            return Response({'err': 3002, 'msg': '上传的文件类型错误，应为jpg、jpeg、gif、png、svg', })

        title = item['title'] = param.get('title', input_filename)

        # ========== 四、获取并处理参数 ===========
        filename = item['filename'] = 'img_' + DigitAlgorithm.make_datetime_17() + '.' + format
        # 上传文件本地保存路径， image是static文件夹下专门存放图片的文件夹
        url = item['url'] = 'upload/images/' + filename
        # print(">>>", request.headers['Host'])

        folder_name = os.path.join(STATICFILES_DIRS[0], 'upload/images/')
        # 没有该目录，则创建
        FileOperate.make_dir(folder_name=folder_name)

        local_path = os.path.join(STATICFILES_DIRS[0], 'upload/images/' + filename)

        with open(local_path, 'wb') as f:
            f.write(input_file.read())
            # print("文件上传完毕")

        item['snapshot'] = {
            'origin_filename': input_filename + '.' + format
        }

        print(">>> make_file_md5:", DigitAlgorithm.make_file_md5(file=local_path))
        item['md5'] = DigitAlgorithm.make_file_md5(file=local_path)
        if not item['md5']:
            return Response({'err': 3003, 'msg': 'local_path不存在', })

        # ========== 五、数据处理 ==========
        serializer = UploadImageSerializer(data=item, context={})
        # 序列化验证，验证失败则获取错误信息
        if not serializer.is_valid():
            print(">>> serializer.errors:", serializer.errors, "\n")
            return Response({'err': 5001, 'msg': serializer.errors, })

        print(">>> item: ", item)

        # 验证成功，保存数据进数据库
        image_instance = serializer.save()

        data_list = {
            'id': image_instance.id,
            'title': title,
            'url': 'http://' + request.headers['Host'] + STATIC_URL + url,
            'path': STATIC_URL + url,
            'filename': filename,
            'format': format,
            'md5': item['md5'],
            # 'thumb': item['thumb'],
            # 'snapshot': snapshot,
        }

        return Response({
            'err': 0,
            'msg': 'OK',
            'data': data_list,
        })










from django.contrib import admin

from .models import ResourceImage, ResourceImageMap, ResourceFile, ResourceFileMap


# Register your models here.


class ResourceImageAdmin(admin.ModelAdmin):
    fields = ('user_id', 'title', 'url', 'filename', 'format', 'md5', 'snapshot', 'thumb')
    list_display = ('id', 'user_id', 'title', 'url', 'filename', 'format', 'md5', 'snapshot')
    search_fields = ('id', 'user_id', 'title', 'url', 'filename', 'format', 'md5', 'snapshot')


class ResourceImageMapAdmin(admin.ModelAdmin):
    fields = ('id', 'image_id', 'source_id', 'source_table', 'price',)
    list_display = ('id', 'image_id', 'source_id', 'source_table', 'price',)
    search_fields = ('id', 'image_id', 'source_id', 'source_table', 'price',)


class ResourceFileAdmin(admin.ModelAdmin):
    fields = ('user_id', 'title', 'url', 'filename', 'format', 'md5', 'snapshot', 'thumb')
    list_display = ('id', 'user_id', 'title', 'url', 'filename', 'format', 'md5', 'snapshot')
    search_fields = ('id', 'user_id', 'title', 'url', 'filename', 'format', 'md5', 'snapshot')


class ResourceFileMapAdmin(admin.ModelAdmin):
    fields = ('id', 'file_id', 'source_id', 'source_table', 'price',)
    list_display = ('id', 'file_id', 'source_id', 'source_table', 'price',)
    search_fields = ('id', 'file_id', 'source_id', 'source_table', 'price',)


admin.site.register(ResourceImage, ResourceImageAdmin)
admin.site.register(ResourceImageMap, ResourceImageMapAdmin)

admin.site.register(ResourceFile, ResourceFileAdmin)
admin.site.register(ResourceFileMap, ResourceFileMapAdmin)

admin.site.site_header = '资源模块'
admin.site.site_title = '资源模块'


from django.contrib import admin
from .models import ResourceImage, ResourceImageMap
# Register your models here.


class ResourceImageAdmin(admin.ModelAdmin):
    fields = ('user_id', 'title', 'url', 'filename', 'format', 'md5', ' thumb', 'snapshot')
    list_display = ('id', 'user_id', 'title', 'url', 'filename', 'format', 'md5', 'thumb', 'snapshot')
    search_fields = ('id', 'user_id', 'title', 'url', 'filename', 'format', 'md5', 'thumb', 'snapshot')


class ResourceImageMapAdmin(admin.ModelAdmin):
    fields = ('id', 'image_id', 'source_id', 'source_table', 'price',)
    list_display = ('id', 'image_id', 'source_id', 'source_table', 'price',)
    search_fields = ('id', 'image_id', 'source_id', 'source_table', 'price',)


admin.site.register(ResourceImage, ResourceImageAdmin)
admin.site.register(ResourceImageMap, ResourceImageMapAdmin)